﻿--[[
	Simple ligthweight AddOn framework
]]

if KarmaLibrary then return; end

KarmaLibrary = {

}

function KarmaLibrary:GetInstance( name, inst )
	local inst = inst or { };
	
	inst._instance = {
		name = name,
		instance = inst,
		libUtility = KarmaUtility,
		libLibrary = KarmaLibrary,
		libAce = KarmaAce,
	}

	inst._addonmeta = {
		title = GetAddOnMetadata( name, "Title" ),
		version = GetAddOnMetadata( name, "Version" ), 
		author = GetAddOnMetadata( name, "Author" ),
		desc = GetAddOnMetadata( name, "Description" ),
		icon = GetAddOnMetadata( name, "X-Icon" ),
		locale = GetLocale() == "enGB" and "enUS" or GetLocale(),
	}
	
	inst._store = {
		db = nil,
		name = nil,
		defaults = { Options = { DebugMode = 0 } },
	}
	inst._options = inst._store.defaults.options
		
	local _inst = inst._instance;
	setmetatable( _inst.libLibrary, { __index = _inst.libUtility } );
	setmetatable( inst, { __index = _inst.libLibrary } );
	
	return inst;
end


function KarmaLibrary:OnEvent( frame, event, ... )
	if event == "VARIABLES_LOADED" then
		self:_DataHookDB();
		if KarmaAce and KarmaAce.OnVariablesLoadedAce then KarmaAce:OnVariablesLoadedAce(frame,self); end
		if self.RunVersionFix then self:RunVersionFix(frame); end;
		if self.OnVariablesLoaded then self:OnVariablesLoaded(frame); return; end
	elseif event == "PLAYER_LOGIN" then
		frame:UnregisterEvent("PLAYER_LOGIN");
		if KarmaAce and KarmaAce.OnPlayerLoginAce then KarmaAce:OnPlayerLoginAce(frame,self); end
		if self.OnPlayerLogin then self:OnPlayerLogin(frame); return; end
	elseif event == "UPDATE_MOUSEOVER_UNIT"	then
		if self.OnUpdateMouse then self:OnUpdateMouse(frame,...); return; end
	end	
	
	if self.OnEventCustom then self:OnEventCustom(frame,event,...) end
end

function KarmaLibrary:_GetDB( name, defaults )
	if not name then return end	
	local db = _G[name] or { };
	_G[name] = db;

	-- init db and chain: db --> defaults ; db(tables) --> defaults(tables)
	for i,v in pairs(defaults) do
		if type(v) == "table"  then
			if not rawget(db, i) then	rawset(db, i, { }) end 
			setmetatable( rawget(db, i), { __index = rawget(defaults, i) } ); 	
		end
	end
	setmetatable( db, { __index = defaults } )
	return db
end

function KarmaLibrary:_DataHookDB( )
	local s = self._store;
	s.db = self:_GetDB(s.name, s.defaults);
	if not s.db then return end
	self._options = s.db.Options
end 

function KarmaLibrary:RegisterDB( name, defaults )
	if not name then return end
	local s = self._store;
	
	s.name = name;
	s.defaults = defaults or s.defaults or { };
					
	self:_DataHookDB( );
end 

function KarmaLibrary:ImportRenameDB(t,new,old)
	if t[old] then 
		t[new] = t[old];
		t[old] = nil;
	end
end

function KarmaLibrary:ImportRenamesDB(t,names)
	for i,v in pairs(names) do
		if type(i) ~= "number" then
			self:ImportRenameDB(t,i,v);
		end
	end
end

function KarmaLibrary:ImportToDB( dbitem, src, prefix, isMerge, names  )
	if not names then return end
	local dst = self._store.db;
	src = src or dst; 
	prefix =  prefix or ""
	
	if dbitem then 
		if not dst[dbitem] then dst[dbitem] = { }  end
		dst = dst[dbitem];
	end

	for i,v in pairs(names) do
		local srcname = prefix..v
		if type(i) == "number" then i = v end
		if src[srcname] then
			if isMerge then
				self:MergeDeep( dst[i], src[srcname], false );
			else
				dst[i] = self:CopyDeep( src[srcname] );
			end
		end
	end
end
